/*
 * libkarma/properties.c
 * 
 * Copyright (c) Frank Zschockelt <libkarma@freakysoft.de> 2004
 * 
 * You may distribute and modify this program under the terms of 
 * the GNU GPL, version 2 or later.
 * 
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "properties.h"
#include "util.h"

struct klist k_list;

void init_properties(void)
{
	k_list.l=NULL;
	k_list.count=0;
	k_list.highest_fid=256;
}

void destroy_properties(void)
{
	struct list * tmp;
	
	while(k_list.l != NULL) {
		tmp=k_list.l;
		k_list.l=tmp->next;
		
		hash_destroy(&(tmp->element));
		free(tmp);
	}

	k_list.count=0;
	k_list.highest_fid=256;
}


uint32_t import_properties(char * properties)
{
	uint32_t count=0;
	char *key, *value;
	char *ikey, *ivalue;
	struct list *item, *l, *o;
	char *n_tok;

	item=malloc(sizeof(struct list));
	item->fid=0;
	item->element=hash_create(32); /* 32 should be nice*/

	key=strtok_r(properties, "=", &n_tok);
	if(key==NULL) return -1;
	value=strtok_r(NULL, "\n", &n_tok);

	while(value != NULL){
		if(strcmp(key, "fid") == 0)
			item->fid=strtoul(value, (char **)NULL, 10);
		else{
			ikey=strdup(key);
			ivalue=strdup(value);
			hash_insert(item->element, ikey, ivalue);
		}
		if((n_tok[0] == '\n')||(n_tok[0]=='\0')){
			++n_tok;
			
			if((k_list.l == NULL) || (k_list.l->fid > item->fid)){
				item->next=k_list.l;
				k_list.l=item;
			}else{
				o=k_list.l;
				l=o->next;
				while((l != NULL) && (l->fid < item->fid)){
					o=l;
					l=l->next;
				}
				item->next=l;
				if(o->fid == item->fid){
					hash_destroy(&o->element);
					o->element=item->element;
					free(item);
				}else
					o->next=item;
			}

			if(item)
				if(item->fid > k_list.highest_fid)
					k_list.highest_fid=item->fid;
			
			item=malloc(sizeof(struct list));
			item->element=hash_create(32);
			++count;
			++k_list.count;
		}
		key=strtok_r(NULL, "=", &n_tok);
		value=strtok_r(NULL, "\n", &n_tok);
	}
	hash_destroy(&item->element);
	free(item);
	return count;
}


static void export_helper(int * length, HASH * src, const char * property, char ** dst)
{
	char *d;

	d=hash_find(src, property);
	if(d!=NULL){
		*length+=strlen(d)+2;
		*length+=strlen(property);
		*dst=realloc(*dst, *length);
		strcat(*dst, property);
		strcat(*dst, "=");
		strcat(*dst, d);
		strcat(*dst, "\n");
	}
}

char * export_property(uint32_t id)
{
	HASH *tmp;
	char *p;
	int len;
	unsigned int i;
	
	static const char * before_fid[] = {
		"artist", "bitrate", "bpm", "codec", "ctime", "duration"};
	static const char * after_fid[] = {
		"fid_generation", "file_id", "genre", "length", "marked", "normalisation", 
		"offset", "options", "play_count", "play_count_limit", "play_last", "profile", 
		"profiler_version", "rid", "rms", "samplerate", "source", "stddev", "title",
		"tracknr", "trailer", "type", "year"};
	
	tmp=idsearch(id);
	if(tmp==NULL) return NULL;
	else{
		len=1;
		p=(char *)malloc(len);
		p[0]='\0';
		
		for(i=0; i < sizeof(before_fid)/sizeof(*before_fid); ++i)
			export_helper(&len, tmp, (char *)before_fid[i], &p);
		/*fid=xxxxxxxxxx\n*/
		len+=strlen(simple_itoa(id))+5;
		p=realloc(p, len);
		strcat(p, "fid=");
		strcat(p, simple_itoa(id));
		strcat(p, "\n");
		
		for(i=0; i < sizeof(after_fid)/sizeof(*after_fid); ++i)
			export_helper(&len, tmp, (char *)after_fid[i], &p);
		
		return p;
	}
}
/*
	The karma only plays files when the ID is > 256 and
	ID%16==0
*/
uint32_t getnextfid(void)
{
	struct list *t;
	uint32_t smallest_fid=272;
	t=k_list.l;
	while((t!=NULL) && (t->fid<=smallest_fid)){
		if(t->fid==smallest_fid)
			smallest_fid+=16;
		t=t->next;
	}
	return smallest_fid;
}

uint32_t new_property(void)
{
	struct list *item, *l;

	item=malloc(sizeof(struct list));
  item->element=hash_create(32);
	item->next=NULL;
	item->fid=getnextfid();

	if((k_list.l == NULL) || (k_list.l->fid > item->fid))
		k_list.l=item;
	else{
		l=k_list.l;
		while((l->next != NULL)&&(l->next->fid < item->fid))
			l=l->next;
		item->next=l->next;
		l->next=item;
	}
	if(item->fid > k_list.highest_fid)
		k_list.highest_fid=item->fid;
	return item->fid;
}


char * get_property(uint32_t fid, char * key)
{
	HASH *tmp;
	
	tmp=idsearch(fid);
	if(tmp==NULL) return NULL;
	else return (char *)hash_find(tmp, key);
}

char * get_property_hash(HASH * p, char * key)
{
	return (char *)hash_find(p, key);
}

int delete_property(uint32_t id)
{
	struct list *l,*p;
	
	l=k_list.l;
	p=k_list.l;
	
	if(p==NULL) return -1;

	while(p->fid != id){
		l=p;
		p=p->next;
	}
	
	if(k_list.l==l)
		k_list.l=p->next;
	else l->next=p->next;

	if(p->fid==k_list.highest_fid)
		if(l!=NULL)
			k_list.highest_fid=l->fid;
	
	hash_destroy(&p->element);
	free(p);
	return 0;
}

int change_property(uint32_t id, char * property, char * data)
{
	HASH * p;
	char *ikey, *ivalue;
	
	p=idsearch(id);
	if(p==NULL) return -1;
	ikey=strdup(property);
	ivalue=strdup(data);
	hash_insert(p, ikey, ivalue);
	
	return 0;
}

void change_property_hash(HASH * p, char * property, char * data)
{
	char *ikey, *ivalue;
	if(data!=NULL){
		ikey=strdup(property);
		ivalue=strdup(data);
		hash_insert(p, ikey, ivalue);
	}
}

HASH * idsearch(uint32_t fid)
{
	struct list * ptr;

	ptr=k_list.l;
	while((ptr!=NULL)&&(ptr->fid < fid))
		ptr=ptr->next;
	if((ptr == NULL) || (ptr->fid != fid)) return NULL;
	else return ptr->element;
}

int bool_search(char * in_prop, char * search_for)
{
	struct list *ptr;
	int hash;
	
	ptr=k_list.l;
	if(ptr!=NULL){
		hash=hash_string(in_prop, ptr->element->nelem);
		while((ptr!=NULL) && (strcmp(hash_find_hash(ptr->element, hash, in_prop), search_for) != 0))
			ptr=ptr->next;
	}
	if(ptr==NULL) return 0;
	else return 1;
}
