/*
 * libkarma/riocp.c
 *
 * Copyright (c) Frank Zschockelt <libkarma@freakysoft.de> 2004
 *
 * You may distribute and modify this program under the terms of 
 * the GNU GPL, version 2 or later.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <ftw.h>

#include "karma.h"
#include "properties.h"
#include "rio_rw.h"

#define CHECK(x) x?"failed":"ok"

void usage(void);

void usage(void)
{
	printf("usage: riocp -a hostname/ip [-p password] [-s filename] [files_to_send ...]\n");
}
	
int main(int argc, char * argv[])
{
	int c;
	//extern char *optarg;
	//extern int optopt, optind;
	char *host=NULL;
	FILE * F=NULL;
	char *p=NULL;
	int karma;
	int treecopy(const char *file, const struct stat *sb, int flag)
	{
		if(flag==FTW_F)
			printf("%s: %s\n", file, CHECK(rio_write(karma, file)));
		return 0;
	}

	
	while ((c = getopt(argc, argv, "ha:p:s:")) != -1) {
		switch(c) {
			case 'h':
				usage();
				return -1;
				break;
			case 'a':
				host=strdup(optarg);
				break;
			case 'p':
				p=strdup(optarg);
				break;
			case 's':
				F=fopen(optarg, "w+");
				break;
			case '?':
				printf("unknown arg %c\n", optopt);
				break;
		}
	}
	
	if(!host){
		usage();
		if(p) free(p);
		return -1;
	}
	
	karma=connectkarma(host);
	free(host);
	
	if(karma < 0){
		usage();
		if(p) free(p);
		return -1;
	}

	init_properties();
	
	if(p){
		printf("authenticate: %i\n", authenticate(karma, p));
		free(p);
	}else printf("authenticate: %i\n", authenticate(karma, ""));
	printf("request_io_lock: %s\n", CHECK(request_io_lock(karma,IO_LOCK_R)));
	
	get_device_settings(karma, &p);
	if(F) fprintf(F,"%s\n", p);
	import_properties(p);
	free(p);
	
	printf("get_all_file_details: %s\n", CHECK(get_all_file_details(karma, &p)));
	if(F){
		fprintf(F,"%s\n", p);
		fclose(F);
	}
	import_properties(p);
	free(p);
  
	if(optind < argc){
		printf("request_io_lock: %s\n", CHECK(request_io_lock(karma,IO_LOCK_W)));
		if(synchronize_necessary(karma)){
			get_all_file_details(karma, &p);
			import_properties(p);
			free(p);
		}
		for(; optind<argc; optind++)
			//printf("%s: %s\n", argv[optind], CHECK(rio_write(karma, argv[optind])));
			ftw(argv[optind], treecopy, 1);

		printf("release_io_lock: %s\n", CHECK(release_io_lock(karma))); 
	}
	printf("hangup: %s\n", CHECK(hangup(karma)));
	close(karma);
	destroy_properties();
	return 0;
}
